"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mapNodeList = mapNodeList;
exports.bindChildren = bindChildren;
exports.translateNode = translateNode;
exports.translateDocument = translateDocument;

var _TText = require("../tree/TText");

var _TPhrasing = require("../tree/TPhrasing");

var _TBlock = require("../tree/TBlock");

var _TDocument = require("../tree/TDocument");

var _toSerializable = require("../dom/to-serializable");

var _TEmpty = require("../tree/TEmpty");

var _HTMLContentModel = _interopRequireDefault(require("../model/HTMLContentModel"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function mapNodeList(nodeList, parentStyles, params) {
  const nextMap = [];

  for (const child of nodeList) {
    const translated = translateNode(child, parentStyles, params);

    if (translated) {
      nextMap.push(translated);
    }
  }

  return nextMap;
}

function bindChildren(node, children, params) {
  node.bindChildren(mapNodeList(children, node.styles, params));
}

function translateElement(node, parentStyles, params) {
  const tagName = node.tagName.toLowerCase();
  const sharedProps = {
    tagName,
    parentStyles,
    stylesMerger: params.stylesMerger,
    attributes: node.attribs
  };

  if (tagName === 'html') {
    const tdoc = new _TDocument.TDocument({ ...sharedProps
    });
    bindChildren(tdoc, node.children, params);
    tdoc.parseChildren();
    return tdoc;
  }

  const model = params.modelRegistry.getElementModelFromTagName(tagName);

  if (!model) {
    return new _TEmpty.TEmpty({ ...sharedProps,
      isUnregistered: true,
      contentModel: null,
      domNode: node
    });
  }

  const contentModel = model.contentModel;

  if (model.isTranslatableTextual()) {
    if (node.children.length === 1) {
      const child = node.children[0];

      if ((0, _toSerializable.isSerializableText)(child)) {
        return new _TText.TText({ ...sharedProps,
          contentModel,
          data: child.data
        });
      }
    }

    const phrasing = new _TPhrasing.TPhrasing({ ...sharedProps,
      contentModel
    });
    bindChildren(phrasing, node.children, params);
    return phrasing;
  }

  if (model.isTranslatableBlock()) {
    const block = new _TBlock.TBlock({ ...sharedProps,
      contentModel,
      parentStyles,
      domChildren: model.isOpaque && !model.isVoid ? node.children : undefined
    });
    bindChildren(block, node.children, params);
    return block;
  }

  return new _TEmpty.TEmpty({ ...sharedProps,
    isUnregistered: false,
    contentModel,
    domNode: node
  });
}

function translateNode(node, parentStyles, params) {
  if ((0, _toSerializable.isSerializableText)(node)) {
    return new _TText.TText({
      data: node.data,
      stylesMerger: params.stylesMerger,
      contentModel: null,
      parentStyles
    });
  }

  if ((0, _toSerializable.isSerializableElement)(node)) {
    return translateElement(node, parentStyles, params);
  }

  return null;
}

function translateDocument(documentTree, params) {
  const serializableDocTree = (0, _toSerializable.toSerializableChildren)(documentTree);
  const rootNodes = mapNodeList(serializableDocTree, params.baseStyles, params);
  let foundTdoc = rootNodes.find(n => n instanceof _TDocument.TDocument);

  if (foundTdoc) {
    return foundTdoc;
  } else {
    const body = new _TBlock.TBlock({
      tagName: 'body',
      stylesMerger: params.stylesMerger,
      contentModel: _HTMLContentModel.default.block,
      parentStyles: null
    });
    body.bindChildren(rootNodes);
    const newTdoc = new _TDocument.TDocument({
      parentStyles: params.baseStyles,
      stylesMerger: params.stylesMerger
    });
    newTdoc.bindChildren([body]);
    return newTdoc;
  }
}
//# sourceMappingURL=translate.js.map